@@ -2,17 +2,18 @@ import threading
 
 from django.http import FileResponse, HttpResponse
 from django.urls import path
+from django.views.decorators.csrf import csrf_exempt
 
 
 def hello(request):
-    name = request.GET.get('name') or 'World'
-    return HttpResponse('Hello %s!' % name)
+    name = request.GET.get("name") or "World"
+    return HttpResponse("Hello %s!" % name)
 
 
 def hello_meta(request):
     return HttpResponse(
-        'From %s' % request.META.get('HTTP_REFERER') or '',
-        content_type=request.META.get('CONTENT_TYPE'),
+        "From %s" % request.META.get("HTTP_REFERER") or "",
+        content_type=request.META.get("CONTENT_TYPE"),
     )
 
 
@@ -23,6 +24,11 @@ def sync_waiter(request):
     return hello(request)
 
 
+@csrf_exempt
+def post_echo(request):
+    return HttpResponse(request.body)
+
+
 sync_waiter.active_threads = set()
 sync_waiter.lock = threading.Lock()
 sync_waiter.barrier = threading.Barrier(2)
@@ -32,8 +38,9 @@ test_filename = __file__
 
 
 urlpatterns = [
-    path('', hello),
-    path('file/', lambda x: FileResponse(open(test_filename, 'rb'))),
-    path('meta/', hello_meta),
-    path('wait/', sync_waiter),
+    path("", hello),
+    path("file/", lambda x: FileResponse(open(test_filename, "rb"))),
+    path("meta/", hello_meta),
+    path("post/", post_echo),
+    path("wait/", sync_waiter),
 ]
