@@ -13,10 +13,22 @@ from django.forms import (
     SplitDateTimeField,
     formsets,
 )
-from django.forms.formsets import BaseFormSet, all_valid, formset_factory
+from django.forms.formsets import (
+    INITIAL_FORM_COUNT,
+    MAX_NUM_FORM_COUNT,
+    MIN_NUM_FORM_COUNT,
+    TOTAL_FORM_COUNT,
+    BaseFormSet,
+    ManagementForm,
+    all_valid,
+    formset_factory,
+)
+from django.forms.renderers import TemplatesSetting, get_default_renderer
 from django.forms.utils import ErrorList
 from django.forms.widgets import HiddenInput
 from django.test import SimpleTestCase
+from django.test.utils import isolate_lru_cache
+from django.utils.deprecation import RemovedInDjango50Warning
 
 from . import jinja2_tests
 
@@ -29,6 +41,12 @@ class Choice(Form):
 ChoiceFormSet = formset_factory(Choice)
 
 
+class ChoiceFormsetWithNonFormError(ChoiceFormSet):
+    def clean(self):
+        super().clean()
+        raise ValidationError("non-form error")
+
+
 class FavoriteDrinkForm(Form):
     name = CharField()
 
@@ -109,8 +127,8 @@ class FormsFormsetTestCase(SimpleTestCase):
 <input type="hidden" name="choices-INITIAL_FORMS" value="0">
 <input type="hidden" name="choices-MIN_NUM_FORMS" value="0">
 <input type="hidden" name="choices-MAX_NUM_FORMS" value="1000">
-<tr><th>Choice:</th><td><input type="text" name="choices-0-choice"></td></tr>
-<tr><th>Votes:</th><td><input type="number" name="choices-0-votes"></td></tr>""",
+<div>Choice:<input type="text" name="choices-0-choice"></div>
+<div>Votes:<input type="number" name="choices-0-votes"></div>""",
         )
         # FormSet are treated similarly to Forms. FormSet has an is_valid()
         # method, and a cleaned_data or errors attribute depending on whether
@@ -389,6 +407,37 @@ class FormsFormsetTestCase(SimpleTestCase):
             '<ul class="errorlist nonform"><li>Please submit at most 1 form.</li></ul>',
         )
 
+    def test_formset_validate_max_flag_custom_error(self):
+        data = {
+            "choices-TOTAL_FORMS": "2",
+            "choices-INITIAL_FORMS": "0",
+            "choices-MIN_NUM_FORMS": "0",
+            "choices-MAX_NUM_FORMS": "2",
+            "choices-0-choice": "Zero",
+            "choices-0-votes": "0",
+            "choices-1-choice": "One",
+            "choices-1-votes": "1",
+        }
+        ChoiceFormSet = formset_factory(Choice, extra=1, max_num=1, validate_max=True)
+        formset = ChoiceFormSet(
+            data,
+            auto_id=False,
+            prefix="choices",
+            error_messages={
+                "too_many_forms": "Number of submitted forms should be at most %(num)d."
+            },
+        )
+        self.assertFalse(formset.is_valid())
+        self.assertEqual(
+            formset.non_form_errors(),
+            ["Number of submitted forms should be at most 1."],
+        )
+        self.assertEqual(
+            str(formset.non_form_errors()),
+            '<ul class="errorlist nonform">'
+            "<li>Number of submitted forms should be at most 1.</li></ul>",
+        )
+
     def test_formset_validate_min_flag(self):
         """
         If validate_min is set and min_num is more than TOTAL_FORMS in the
@@ -416,6 +465,37 @@ class FormsFormsetTestCase(SimpleTestCase):
             "Please submit at least 3 forms.</li></ul>",
         )
 
+    def test_formset_validate_min_flag_custom_formatted_error(self):
+        data = {
+            "choices-TOTAL_FORMS": "2",
+            "choices-INITIAL_FORMS": "0",
+            "choices-MIN_NUM_FORMS": "0",
+            "choices-MAX_NUM_FORMS": "0",
+            "choices-0-choice": "Zero",
+            "choices-0-votes": "0",
+            "choices-1-choice": "One",
+            "choices-1-votes": "1",
+        }
+        ChoiceFormSet = formset_factory(Choice, extra=1, min_num=3, validate_min=True)
+        formset = ChoiceFormSet(
+            data,
+            auto_id=False,
+            prefix="choices",
+            error_messages={
+                "too_few_forms": "Number of submitted forms should be at least %(num)d."
+            },
+        )
+        self.assertFalse(formset.is_valid())
+        self.assertEqual(
+            formset.non_form_errors(),
+            ["Number of submitted forms should be at least 3."],
+        )
+        self.assertEqual(
+            str(formset.non_form_errors()),
+            '<ul class="errorlist nonform">'
+            "<li>Number of submitted forms should be at least 3.</li></ul>",
+        )
+
     def test_formset_validate_min_unchanged_forms(self):
         """
         min_num validation doesn't consider unchanged forms with initial data
@@ -907,12 +987,12 @@ class FormsFormsetTestCase(SimpleTestCase):
         formset = LimitedFavoriteDrinkFormSet()
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
-            """<tr><th><label for="id_form-0-name">Name:</label></th>
-<td><input type="text" name="form-0-name" id="id_form-0-name"></td></tr>
-<tr><th><label for="id_form-1-name">Name:</label></th>
-<td><input type="text" name="form-1-name" id="id_form-1-name"></td></tr>
-<tr><th><label for="id_form-2-name">Name:</label></th>
-<td><input type="text" name="form-2-name" id="id_form-2-name"></td></tr>""",
+            """<div><label for="id_form-0-name">Name:</label>
+            <input type="text" name="form-0-name" id="id_form-0-name"></div>
+<div><label for="id_form-1-name">Name:</label>
+<input type="text" name="form-1-name" id="id_form-1-name"></div>
+<div><label for="id_form-2-name">Name:</label>
+<input type="text" name="form-2-name" id="id_form-2-name"></div>""",
         )
         # If max_num is 0 then no form is rendered at all.
         LimitedFavoriteDrinkFormSet = formset_factory(
@@ -928,10 +1008,10 @@ class FormsFormsetTestCase(SimpleTestCase):
         formset = LimitedFavoriteDrinkFormSet()
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
-            """<tr><th><label for="id_form-0-name">Name:</label></th><td>
-<input type="text" name="form-0-name" id="id_form-0-name"></td></tr>
-<tr><th><label for="id_form-1-name">Name:</label></th>
-<td><input type="text" name="form-1-name" id="id_form-1-name"></td></tr>""",
+            """<div><label for="id_form-0-name">Name:</label>
+<input type="text" name="form-0-name" id="id_form-0-name"></div>
+<div><label for="id_form-1-name">Name:</label>
+<input type="text" name="form-1-name" id="id_form-1-name"></div>""",
         )
 
     def test_limiting_extra_lest_than_max_num(self):
@@ -942,8 +1022,8 @@ class FormsFormsetTestCase(SimpleTestCase):
         formset = LimitedFavoriteDrinkFormSet()
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
-            """<tr><th><label for="id_form-0-name">Name:</label></th>
-<td><input type="text" name="form-0-name" id="id_form-0-name"></td></tr>""",
+            """<div><label for="id_form-0-name">Name:</label>
+<input type="text" name="form-0-name" id="id_form-0-name"></div>""",
         )
 
     def test_max_num_with_initial_data(self):
@@ -955,11 +1035,11 @@ class FormsFormsetTestCase(SimpleTestCase):
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
             """
-            <tr><th><label for="id_form-0-name">Name:</label></th>
-            <td><input type="text" name="form-0-name" value="Fernet and Coke"
-                id="id_form-0-name"></td></tr>
-            <tr><th><label for="id_form-1-name">Name:</label></th>
-            <td><input type="text" name="form-1-name" id="id_form-1-name"></td></tr>
+            <div><label for="id_form-0-name">Name:</label>
+            <input type="text" name="form-0-name" value="Fernet and Coke"
+                id="id_form-0-name"></div>
+            <div><label for="id_form-1-name">Name:</label>
+            <input type="text" name="form-1-name" id="id_form-1-name"></div>
             """,
         )
 
@@ -987,12 +1067,12 @@ class FormsFormsetTestCase(SimpleTestCase):
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
             """
-            <tr><th><label for="id_form-0-name">Name:</label></th>
-            <td><input id="id_form-0-name" name="form-0-name" type="text"
-                value="Fernet and Coke"></td></tr>
-            <tr><th><label for="id_form-1-name">Name:</label></th>
-            <td><input id="id_form-1-name" name="form-1-name" type="text"
-                value="Bloody Mary"></td></tr>
+            <div><label for="id_form-0-name">Name:</label>
+            <input id="id_form-0-name" name="form-0-name" type="text"
+                value="Fernet and Coke"></div>
+            <div><label for="id_form-1-name">Name:</label>
+            <input id="id_form-1-name" name="form-1-name" type="text"
+                value="Bloody Mary"></div>
             """,
         )
 
@@ -1013,18 +1093,15 @@ class FormsFormsetTestCase(SimpleTestCase):
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
             """
-            <tr><th><label for="id_form-0-name">Name:</label></th>
-            <td>
+            <div><label for="id_form-0-name">Name:</label>
             <input id="id_form-0-name" name="form-0-name" type="text" value="Gin Tonic">
-            </td></tr>
-            <tr><th><label for="id_form-1-name">Name:</label></th>
-            <td>
+            </div>
+            <div><label for="id_form-1-name">Name:</label>
             <input id="id_form-1-name" name="form-1-name" type="text"
-                value="Bloody Mary"></td></tr>
-            <tr><th><label for="id_form-2-name">Name:</label></th>
-            <td>
+                value="Bloody Mary"></div>
+            <div><label for="id_form-2-name">Name:</label>
             <input id="id_form-2-name" name="form-2-name" type="text"
-                value="Jack and Coke"></td></tr>
+                value="Jack and Coke"></div>
             """,
         )
 
@@ -1104,12 +1181,23 @@ class FormsFormsetTestCase(SimpleTestCase):
         self.assertHTMLEqual(
             "\n".join(str(form) for form in formset.forms),
             """
-            <tr><th><label for="id_form-0-name">Name:</label></th>
-            <td>
+            <div><label for="id_form-0-name">Name:</label>
             <input type="text" name="form-0-name" value="Gin Tonic" id="id_form-0-name">
-            </td></tr>
-            <tr><th><label for="id_form-1-name">Name:</label></th>
-            <td><input type="text" name="form-1-name" id="id_form-1-name"></td></tr>""",
+            </div>
+            <div><label for="id_form-1-name">Name:</label>
+            <input type="text" name="form-1-name" id="id_form-1-name"></div>""",
+        )
+
+    def test_management_form_field_names(self):
+        """The management form class has field names matching the constants."""
+        self.assertCountEqual(
+            ManagementForm.base_fields,
+            [
+                TOTAL_FORM_COUNT,
+                INITIAL_FORM_COUNT,
+                MIN_NUM_FORM_COUNT,
+                MAX_NUM_FORM_COUNT,
+            ],
         )
 
     def test_management_form_prefix(self):
@@ -1417,6 +1505,26 @@ class FormsFormsetTestCase(SimpleTestCase):
         self.assertIs(formset._should_delete_form(formset.forms[1]), False)
         self.assertIs(formset._should_delete_form(formset.forms[2]), False)
 
+    def test_template_name_uses_renderer_value(self):
+        class CustomRenderer(TemplatesSetting):
+            formset_template_name = "a/custom/formset/template.html"
+
+        ChoiceFormSet = formset_factory(Choice, renderer=CustomRenderer)
+
+        self.assertEqual(
+            ChoiceFormSet().template_name, "a/custom/formset/template.html"
+        )
+
+    def test_template_name_can_be_overridden(self):
+        class CustomFormSet(BaseFormSet):
+            template_name = "a/custom/formset/template.html"
+
+        ChoiceFormSet = formset_factory(Choice, formset=CustomFormSet)
+
+        self.assertEqual(
+            ChoiceFormSet().template_name, "a/custom/formset/template.html"
+        )
+
     def test_custom_renderer(self):
         """
         A custom renderer passed to a formset_factory() is passed to all forms
@@ -1442,6 +1550,63 @@ class FormsFormsetTestCase(SimpleTestCase):
         self.assertEqual(formset.non_form_errors().renderer, renderer)
         self.assertEqual(formset.empty_form.renderer, renderer)
 
+    def test_repr(self):
+        valid_formset = self.make_choiceformset([("test", 1)])
+        valid_formset.full_clean()
+        invalid_formset = self.make_choiceformset([("test", "")])
+        invalid_formset.full_clean()
+        partially_invalid_formset = self.make_choiceformset(
+            [("test", "1"), ("test", "")],
+        )
+        partially_invalid_formset.full_clean()
+        invalid_formset_non_form_errors_only = self.make_choiceformset(
+            [("test", "")],
+            formset_class=ChoiceFormsetWithNonFormError,
+        )
+        invalid_formset_non_form_errors_only.full_clean()
+
+        cases = [
+            (
+                self.make_choiceformset(),
+                "<ChoiceFormSet: bound=False valid=Unknown total_forms=1>",
+            ),
+            (
+                self.make_choiceformset(
+                    formset_class=formset_factory(Choice, extra=10),
+                ),
+                "<ChoiceFormSet: bound=False valid=Unknown total_forms=10>",
+            ),
+            (
+                self.make_choiceformset([]),
+                "<ChoiceFormSet: bound=True valid=Unknown total_forms=0>",
+            ),
+            (
+                self.make_choiceformset([("test", 1)]),
+                "<ChoiceFormSet: bound=True valid=Unknown total_forms=1>",
+            ),
+            (valid_formset, "<ChoiceFormSet: bound=True valid=True total_forms=1>"),
+            (invalid_formset, "<ChoiceFormSet: bound=True valid=False total_forms=1>"),
+            (
+                partially_invalid_formset,
+                "<ChoiceFormSet: bound=True valid=False total_forms=2>",
+            ),
+            (
+                invalid_formset_non_form_errors_only,
+                "<ChoiceFormsetWithNonFormError: bound=True valid=False total_forms=1>",
+            ),
+        ]
+        for formset, expected_repr in cases:
+            with self.subTest(expected_repr=expected_repr):
+                self.assertEqual(repr(formset), expected_repr)
+
+    def test_repr_do_not_trigger_validation(self):
+        formset = self.make_choiceformset([("test", 1)])
+        with mock.patch.object(formset, "full_clean") as mocked_full_clean:
+            repr(formset)
+            mocked_full_clean.assert_not_called()
+            formset.is_valid()
+            mocked_full_clean.assert_called()
+
 
 @jinja2_tests
 class Jinja2FormsFormsetTestCase(FormsFormsetTestCase):
@@ -1501,6 +1666,18 @@ class FormsetAsTagTests(SimpleTestCase):
             ),
         )
 
+    def test_as_div(self):
+        self.assertHTMLEqual(
+            self.formset.as_div(),
+            self.management_form_html
+            + (
+                "<div>Choice: "
+                '<input type="text" name="choices-0-choice" value="Calexico"></div>'
+                '<div>Votes: <input type="number" name="choices-0-votes" value="100">'
+                "</div>"
+            ),
+        )
+
 
 @jinja2_tests
 class Jinja2FormsetAsTagTests(FormsetAsTagTests):
@@ -1531,16 +1708,16 @@ class TestIsBoundBehavior(SimpleTestCase):
         # Can still render the formset.
         self.assertHTMLEqual(
             str(formset),
-            '<tr><td colspan="2">'
             '<ul class="errorlist nonfield">'
             "<li>(Hidden field TOTAL_FORMS) This field is required.</li>"
             "<li>(Hidden field INITIAL_FORMS) This field is required.</li>"
             "</ul>"
+            "<div>"
             '<input type="hidden" name="form-TOTAL_FORMS" id="id_form-TOTAL_FORMS">'
             '<input type="hidden" name="form-INITIAL_FORMS" id="id_form-INITIAL_FORMS">'
             '<input type="hidden" name="form-MIN_NUM_FORMS" id="id_form-MIN_NUM_FORMS">'
             '<input type="hidden" name="form-MAX_NUM_FORMS" id="id_form-MAX_NUM_FORMS">'
-            "</td></tr>\n",
+            "</div>\n",
         )
 
     def test_management_form_invalid_data(self):
@@ -1562,18 +1739,18 @@ class TestIsBoundBehavior(SimpleTestCase):
         # Can still render the formset.
         self.assertHTMLEqual(
             str(formset),
-            '<tr><td colspan="2">'
             '<ul class="errorlist nonfield">'
             "<li>(Hidden field TOTAL_FORMS) Enter a whole number.</li>"
             "<li>(Hidden field INITIAL_FORMS) Enter a whole number.</li>"
             "</ul>"
+            "<div>"
             '<input type="hidden" name="form-TOTAL_FORMS" value="two" '
             'id="id_form-TOTAL_FORMS">'
             '<input type="hidden" name="form-INITIAL_FORMS" value="one" '
             'id="id_form-INITIAL_FORMS">'
             '<input type="hidden" name="form-MIN_NUM_FORMS" id="id_form-MIN_NUM_FORMS">'
             '<input type="hidden" name="form-MAX_NUM_FORMS" id="id_form-MAX_NUM_FORMS">'
-            "</td></tr>\n",
+            "</div>\n",
         )
 
     def test_customize_management_form_error(self):
@@ -1719,3 +1896,28 @@ class AllValidTests(SimpleTestCase):
         ]
         self.assertEqual(formset1._errors, expected_errors)
         self.assertEqual(formset2._errors, expected_errors)
+
+
+class DeprecationTests(SimpleTestCase):
+    def test_warning(self):
+        from django.forms.utils import DEFAULT_TEMPLATE_DEPRECATION_MSG
+
+        with isolate_lru_cache(get_default_renderer), self.settings(
+            FORM_RENDERER="django.forms.renderers.DjangoTemplates"
+        ), self.assertRaisesMessage(
+            RemovedInDjango50Warning, DEFAULT_TEMPLATE_DEPRECATION_MSG
+        ):
+            ChoiceFormSet = formset_factory(Choice)
+            formset = ChoiceFormSet()
+            str(formset)
+
+    def test_no_management_form_warning(self):
+        """
+        Management forms are already rendered with the new div template.
+        """
+        with isolate_lru_cache(get_default_renderer), self.settings(
+            FORM_RENDERER="django.forms.renderers.DjangoTemplates"
+        ):
+            ChoiceFormSet = formset_factory(Choice, formset=BaseFormSet)
+            formset = ChoiceFormSet()
+            str(formset.management_form)
