@@ -865,11 +865,12 @@ class FormattingTests(SimpleTestCase):
         self.maxDiff = 3000
         # Catalan locale
         with translation.override("ca", deactivate=True):
-            self.assertEqual(r"j \d\e F \d\e Y", get_format("DATE_FORMAT"))
+            self.assertEqual(r"j E \d\e Y", get_format("DATE_FORMAT"))
             self.assertEqual(1, get_format("FIRST_DAY_OF_WEEK"))
             self.assertEqual(",", get_format("DECIMAL_SEPARATOR"))
             self.assertEqual("10:15", time_format(self.t))
-            self.assertEqual("31 de desembre de 2009", date_format(self.d))
+            self.assertEqual("31 desembre de 2009", date_format(self.d))
+            self.assertEqual("1 abril de 2009", date_format(datetime.date(2009, 4, 1)))
             self.assertEqual(
                 "desembre del 2009", date_format(self.d, "YEAR_MONTH_FORMAT")
             )
@@ -888,10 +889,8 @@ class FormattingTests(SimpleTestCase):
                 self.assertEqual("66666,666", localize(self.n))
                 self.assertEqual("99999,999", localize(self.f))
                 self.assertEqual("10000", localize(self.long))
-                self.assertEqual("31 de desembre de 2009", localize(self.d))
-                self.assertEqual(
-                    "31 de desembre de 2009 a les 20:50", localize(self.dt)
-                )
+                self.assertEqual("31 desembre de 2009", localize(self.d))
+                self.assertEqual("31 desembre de 2009 a les 20:50", localize(self.dt))
 
             with self.settings(USE_THOUSAND_SEPARATOR=True):
                 self.assertEqual("66.666,666", Template("{{ n }}").render(self.ctxt))
@@ -930,10 +929,10 @@ class FormattingTests(SimpleTestCase):
                 self.assertEqual("66666,666", Template("{{ n }}").render(self.ctxt))
                 self.assertEqual("99999,999", Template("{{ f }}").render(self.ctxt))
                 self.assertEqual(
-                    "31 de desembre de 2009", Template("{{ d }}").render(self.ctxt)
+                    "31 desembre de 2009", Template("{{ d }}").render(self.ctxt)
                 )
                 self.assertEqual(
-                    "31 de desembre de 2009 a les 20:50",
+                    "31 desembre de 2009 a les 20:50",
                     Template("{{ dt }}").render(self.ctxt),
                 )
                 self.assertEqual(
@@ -1316,7 +1315,7 @@ class FormattingTests(SimpleTestCase):
             with translation.override("de-at", deactivate=True):
                 self.assertEqual("66.666,666", Template("{{ n }}").render(self.ctxt))
             with translation.override("es-us", deactivate=True):
-                self.assertEqual("31 de Diciembre de 2009", date_format(self.d))
+                self.assertEqual("31 de diciembre de 2009", date_format(self.d))
 
     def test_localized_input(self):
         """
@@ -1515,6 +1514,9 @@ class FormattingTests(SimpleTestCase):
         with translation.override("de", deactivate=True):
             self.assertEqual(".", get_format("DECIMAL_SEPARATOR", lang="en"))
 
+    def test_get_format_lazy_format(self):
+        self.assertEqual(get_format(gettext_lazy("DATE_FORMAT")), "N j, Y")
+
     def test_localize_templatetag_and_filter(self):
         """
         Test the {% localize %} templatetag and the localize/unlocalize filters.
@@ -1728,14 +1730,6 @@ class MiscTests(SimpleTestCase):
             ("de;q=0.", [("de", 0.0)]),
             ("en; q=1,", [("en", 1.0)]),
             ("en; q=1.0, * ; q=0.5", [("en", 1.0), ("*", 0.5)]),
-            (
-                "en" + "-x" * 20,
-                [("en-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x-x", 1.0)],
-            ),
-            (
-                ", ".join(["en; q=1.0"] * 20),
-                [("en", 1.0)] * 20,
-            ),
             # Bad headers
             ("en-gb;q=1.0000", []),
             ("en;q=0.1234", []),
@@ -1752,10 +1746,6 @@ class MiscTests(SimpleTestCase):
             ("", []),
             ("en;q=1e0", []),
             ("en-au;q=１.０", []),
-            # Invalid as language-range value too long.
-            ("xxxxxxxx" + "-xxxxxxxx" * 500, []),
-            # Header value too long, only parse up to limit.
-            (", ".join(["en; q=1.0"] * 500), [("en", 1.0)] * 45),
         ]
         for value, expected in tests:
             with self.subTest(value=value):
@@ -1908,9 +1898,10 @@ class MiscTests(SimpleTestCase):
         USE_I18N=True,
         LANGUAGES=[
             ("en", "English"),
+            ("ar-dz", "Algerian Arabic"),
             ("de", "German"),
             ("de-at", "Austrian German"),
-            ("pt-br", "Portuguese (Brazil)"),
+            ("pt-BR", "Portuguese (Brazil)"),
         ],
     )
     def test_get_supported_language_variant_real(self):
@@ -1921,8 +1912,11 @@ class MiscTests(SimpleTestCase):
         self.assertEqual(g("de-at"), "de-at")
         self.assertEqual(g("de-ch"), "de")
         self.assertEqual(g("pt-br"), "pt-br")
+        self.assertEqual(g("pt-BR"), "pt-BR")
         self.assertEqual(g("pt"), "pt-br")
         self.assertEqual(g("pt-pt"), "pt-br")
+        self.assertEqual(g("ar-dz"), "ar-dz")
+        self.assertEqual(g("ar-DZ"), "ar-DZ")
         with self.assertRaises(LookupError):
             g("pt", strict=True)
         with self.assertRaises(LookupError):
@@ -1951,22 +1945,41 @@ class MiscTests(SimpleTestCase):
     @override_settings(
         LANGUAGES=[
             ("en", "English"),
+            ("en-latn-us", "Latin English"),
             ("de", "German"),
+            ("de-1996", "German, orthography of 1996"),
             ("de-at", "Austrian German"),
+            ("de-ch-1901", "German, Swiss variant, traditional orthography"),
+            ("i-mingo", "Mingo"),
+            ("kl-tunumiit", "Tunumiisiut"),
+            ("nan-hani-tw", "Hanji"),
             ("pl", "Polish"),
         ],
     )
     def test_get_language_from_path_real(self):
         g = trans_real.get_language_from_path
-        self.assertEqual(g("/pl/"), "pl")
-        self.assertEqual(g("/pl"), "pl")
-        self.assertIsNone(g("/xyz/"))
-        self.assertEqual(g("/en/"), "en")
-        self.assertEqual(g("/en-gb/"), "en")
-        self.assertEqual(g("/de/"), "de")
-        self.assertEqual(g("/de-at/"), "de-at")
-        self.assertEqual(g("/de-ch/"), "de")
-        self.assertIsNone(g("/de-simple-page/"))
+        tests = [
+            ("/pl/", "pl"),
+            ("/pl", "pl"),
+            ("/xyz/", None),
+            ("/en/", "en"),
+            ("/en-gb/", "en"),
+            ("/en-latn-us/", "en-latn-us"),
+            ("/en-Latn-US/", "en-Latn-US"),
+            ("/de/", "de"),
+            ("/de-1996/", "de-1996"),
+            ("/de-at/", "de-at"),
+            ("/de-AT/", "de-AT"),
+            ("/de-ch/", "de"),
+            ("/de-ch-1901/", "de-ch-1901"),
+            ("/de-simple-page-test/", None),
+            ("/i-mingo/", "i-mingo"),
+            ("/kl-tunumiit/", "kl-tunumiit"),
+            ("/nan-hani-tw/", "nan-hani-tw"),
+        ]
+        for path, language in tests:
+            with self.subTest(path=path):
+                self.assertEqual(g(path), language)
 
     def test_get_language_from_path_null(self):
         g = trans_null.get_language_from_path
@@ -2168,7 +2181,7 @@ class UnprefixedDefaultLanguageTests(SimpleTestCase):
 
     def test_page_with_dash(self):
         # A page starting with /de* shouldn't match the 'de' language code.
-        response = self.client.get("/de-simple-page/")
+        response = self.client.get("/de-simple-page-test/")
         self.assertEqual(response.content, b"Yes")
 
     def test_no_redirect_on_404(self):
