@@ -253,6 +253,16 @@ class InspectDBTestCase(TestCase):
             output,
         )
 
+    @skipUnlessDBFeature("can_introspect_foreign_keys")
+    def test_foreign_key_to_field(self):
+        out = StringIO()
+        call_command("inspectdb", "inspectdb_foreignkeytofield", stdout=out)
+        self.assertIn(
+            "to_field_fk = models.ForeignKey('InspectdbPeoplemoredata', "
+            "models.DO_NOTHING, to_field='people_unique_id')",
+            out.getvalue(),
+        )
+
     def test_digits_column_name_introspection(self):
         """Introspection of column names consist/start with digits (#16536/#17676)"""
         char_field_type = connection.features.introspected_field_types["CharField"]
@@ -320,6 +330,13 @@ class InspectDBTestCase(TestCase):
         output = out.getvalue()
         self.assertIn("class InspectdbSpecialTableName(models.Model):", output)
 
+    @skipUnlessDBFeature("supports_expression_indexes")
+    def test_table_with_func_unique_constraint(self):
+        out = StringIO()
+        call_command("inspectdb", "inspectdb_funcuniqueconstraint", stdout=out)
+        output = out.getvalue()
+        self.assertIn("class InspectdbFuncuniqueconstraint(models.Model):", output)
+
     def test_managed_models(self):
         """
         By default the command generates models with `Meta.managed = False`.
