@@ -29,10 +29,11 @@ from django.core.validators import EmailValidator, RegexValidator
 from django.db import migrations, models
 from django.db.migrations.serializer import BaseSerializer
 from django.db.migrations.writer import MigrationWriter, OperationWriter
-from django.test import SimpleTestCase
+from django.test import SimpleTestCase, ignore_warnings
 from django.utils.deconstruct import deconstructible
+from django.utils.deprecation import RemovedInDjango50Warning
 from django.utils.functional import SimpleLazyObject
-from django.utils.timezone import get_default_timezone, get_fixed_timezone, utc
+from django.utils.timezone import get_default_timezone, get_fixed_timezone
 from django.utils.translation import gettext_lazy as _
 
 from .models import FoodManager, FoodQuerySet
@@ -532,13 +533,16 @@ class WriterTests(SimpleTestCase):
             datetime.datetime(2014, 1, 1, 1, 1),
             ("datetime.datetime(2014, 1, 1, 1, 1)", {"import datetime"}),
         )
+        with ignore_warnings(category=RemovedInDjango50Warning):
+            from django.utils.timezone import utc
         for tzinfo in (utc, datetime.timezone.utc):
             with self.subTest(tzinfo=tzinfo):
                 self.assertSerializedResultEqual(
                     datetime.datetime(2012, 1, 1, 1, 1, tzinfo=tzinfo),
                     (
-                        "datetime.datetime(2012, 1, 1, 1, 1, tzinfo=utc)",
-                        {"import datetime", "from django.utils.timezone import utc"},
+                        "datetime.datetime"
+                        "(2012, 1, 1, 1, 1, tzinfo=datetime.timezone.utc)",
+                        {"import datetime"},
                     ),
                 )
 
@@ -547,8 +551,8 @@ class WriterTests(SimpleTestCase):
                 2012, 1, 1, 2, 1, tzinfo=zoneinfo.ZoneInfo("Europe/Paris")
             ),
             (
-                "datetime.datetime(2012, 1, 1, 1, 1, tzinfo=utc)",
-                {"import datetime", "from django.utils.timezone import utc"},
+                "datetime.datetime(2012, 1, 1, 1, 1, tzinfo=datetime.timezone.utc)",
+                {"import datetime"},
             ),
         )
         if pytz:
@@ -557,8 +561,8 @@ class WriterTests(SimpleTestCase):
                     datetime.datetime(2012, 1, 1, 2, 1)
                 ),
                 (
-                    "datetime.datetime(2012, 1, 1, 1, 1, tzinfo=utc)",
-                    {"import datetime", "from django.utils.timezone import utc"},
+                    "datetime.datetime(2012, 1, 1, 1, 1, tzinfo=datetime.timezone.utc)",
+                    {"import datetime"},
                 ),
             )
 
@@ -670,6 +674,33 @@ class WriterTests(SimpleTestCase):
         ):
             MigrationWriter.serialize(validator)
 
+    def test_serialize_complex_func_index(self):
+        index = models.Index(
+            models.Func("rating", function="ABS"),
+            models.Case(
+                models.When(name="special", then=models.Value("X")),
+                default=models.Value("other"),
+            ),
+            models.ExpressionWrapper(
+                models.F("pages"),
+                output_field=models.IntegerField(),
+            ),
+            models.OrderBy(models.F("name").desc()),
+            name="complex_func_index",
+        )
+        string, imports = MigrationWriter.serialize(index)
+        self.assertEqual(
+            string,
+            "models.Index(models.Func('rating', function='ABS'), "
+            "models.Case(models.When(name='special', then=models.Value('X')), "
+            "default=models.Value('other')), "
+            "models.ExpressionWrapper("
+            "models.F('pages'), output_field=models.IntegerField()), "
+            "models.OrderBy(models.OrderBy(models.F('name'), descending=True)), "
+            "name='complex_func_index')",
+        )
+        self.assertEqual(imports, {"from django.db import models"})
+
     def test_serialize_empty_nonempty_tuple(self):
         """
         Ticket #22679: makemigrations generates invalid code for (an empty
@@ -861,7 +892,9 @@ class WriterTests(SimpleTestCase):
                         "mymodel",
                         "myfield",
                         models.DateTimeField(
-                            default=datetime.datetime(2012, 1, 1, 1, 1, tzinfo=utc),
+                            default=datetime.datetime(
+                                2012, 1, 1, 1, 1, tzinfo=datetime.timezone.utc
+                            ),
                         ),
                     ),
                 ]
@@ -870,9 +903,7 @@ class WriterTests(SimpleTestCase):
         writer = MigrationWriter(migration)
         output = writer.as_string()
         self.assertIn(
-            "import datetime\n"
-            "from django.db import migrations, models\n"
-            "from django.utils.timezone import utc\n",
+            "import datetime\nfrom django.db import migrations, models\n",
             output,
         )
 
@@ -881,7 +912,7 @@ class WriterTests(SimpleTestCase):
         Test comments at top of file.
         """
         migration = type("Migration", (migrations.Migration,), {"operations": []})
-        dt = datetime.datetime(2015, 7, 31, 4, 40, 0, 0, tzinfo=utc)
+        dt = datetime.datetime(2015, 7, 31, 4, 40, 0, 0, tzinfo=datetime.timezone.utc)
         with mock.patch("django.db.migrations.writer.now", lambda: dt):
             for include_header in (True, False):
                 with self.subTest(include_header=include_header):
