@@ -55,6 +55,41 @@ class BasicTests(PostgreSQLSimpleTestCase):
                 instance = Model(field=value)
                 self.assertEqual(instance.get_field_display(), display)
 
+    def test_discrete_range_fields_unsupported_default_bounds(self):
+        discrete_range_types = [
+            pg_fields.BigIntegerRangeField,
+            pg_fields.IntegerRangeField,
+            pg_fields.DateRangeField,
+        ]
+        for field_type in discrete_range_types:
+            msg = f"Cannot use 'default_bounds' with {field_type.__name__}."
+            with self.assertRaisesMessage(TypeError, msg):
+                field_type(choices=[((51, 100), "51-100")], default_bounds="[]")
+
+    def test_continuous_range_fields_default_bounds(self):
+        continuous_range_types = [
+            pg_fields.DecimalRangeField,
+            pg_fields.DateTimeRangeField,
+        ]
+        for field_type in continuous_range_types:
+            field = field_type(choices=[((51, 100), "51-100")], default_bounds="[]")
+            self.assertEqual(field.default_bounds, "[]")
+
+    def test_invalid_default_bounds(self):
+        tests = [")]", ")[", "](", "])", "([", "[(", "x", "", None]
+        msg = "default_bounds must be one of '[)', '(]', '()', or '[]'."
+        for invalid_bounds in tests:
+            with self.assertRaisesMessage(ValueError, msg):
+                pg_fields.DecimalRangeField(default_bounds=invalid_bounds)
+
+    def test_deconstruct(self):
+        field = pg_fields.DecimalRangeField()
+        *_, kwargs = field.deconstruct()
+        self.assertEqual(kwargs, {})
+        field = pg_fields.DecimalRangeField(default_bounds="[]")
+        *_, kwargs = field.deconstruct()
+        self.assertEqual(kwargs, {"default_bounds": "[]"})
+
 
 class TestSaveLoad(PostgreSQLTestCase):
     def test_all_fields(self):
@@ -87,6 +122,19 @@ class TestSaveLoad(PostgreSQLTestCase):
         loaded = RangesModel.objects.get()
         self.assertEqual(NumericRange(0, 10), loaded.ints)
 
+    def test_tuple_range_with_default_bounds(self):
+        range_ = (timezone.now(), timezone.now() + datetime.timedelta(hours=1))
+        RangesModel.objects.create(timestamps_closed_bounds=range_, timestamps=range_)
+        loaded = RangesModel.objects.get()
+        self.assertEqual(
+            loaded.timestamps_closed_bounds,
+            DateTimeTZRange(range_[0], range_[1], "[]"),
+        )
+        self.assertEqual(
+            loaded.timestamps,
+            DateTimeTZRange(range_[0], range_[1], "[)"),
+        )
+
     def test_range_object_boundaries(self):
         r = NumericRange(0, 10, "[]")
         instance = RangesModel(decimals=r)
@@ -95,6 +143,16 @@ class TestSaveLoad(PostgreSQLTestCase):
         self.assertEqual(r, loaded.decimals)
         self.assertIn(10, loaded.decimals)
 
+    def test_range_object_boundaries_range_with_default_bounds(self):
+        range_ = DateTimeTZRange(
+            timezone.now(),
+            timezone.now() + datetime.timedelta(hours=1),
+            bounds="()",
+        )
+        RangesModel.objects.create(timestamps_closed_bounds=range_)
+        loaded = RangesModel.objects.get()
+        self.assertEqual(loaded.timestamps_closed_bounds, range_)
+
     def test_unbounded(self):
         r = NumericRange(None, None, "()")
         instance = RangesModel(decimals=r)
@@ -496,31 +554,38 @@ class TestSerialization(PostgreSQLSimpleTestCase):
         '"{\\"upper\\": \\"2014-02-02T12:12:12+00:00\\", '
         '\\"lower\\": \\"2014-01-01T00:00:00+00:00\\", \\"bounds\\": \\"[)\\"}", '
         '"timestamps_inner": null, '
+        '"timestamps_closed_bounds": "{\\"upper\\": \\"2014-02-02T12:12:12+00:00\\", '
+        '\\"lower\\": \\"2014-01-01T00:00:00+00:00\\", \\"bounds\\": \\"()\\"}", '
         '"dates": "{\\"upper\\": \\"2014-02-02\\", \\"lower\\": \\"2014-01-01\\", '
-        '\\"bounds\\": \\"[)\\"}", '
-        '"dates_inner": null }, '
+        '\\"bounds\\": \\"[)\\"}", "dates_inner": null }, '
         '"model": "postgres_tests.rangesmodel", "pk": null}]'
     )
 
     lower_date = datetime.date(2014, 1, 1)
     upper_date = datetime.date(2014, 2, 2)
-    lower_dt = datetime.datetime(2014, 1, 1, 0, 0, 0, tzinfo=timezone.utc)
-    upper_dt = datetime.datetime(2014, 2, 2, 12, 12, 12, tzinfo=timezone.utc)
+    lower_dt = datetime.datetime(2014, 1, 1, 0, 0, 0, tzinfo=datetime.timezone.utc)
+    upper_dt = datetime.datetime(2014, 2, 2, 12, 12, 12, tzinfo=datetime.timezone.utc)
 
     def test_dumping(self):
         instance = RangesModel(
             ints=NumericRange(0, 10),
             decimals=NumericRange(empty=True),
             timestamps=DateTimeTZRange(self.lower_dt, self.upper_dt),
+            timestamps_closed_bounds=DateTimeTZRange(
+                self.lower_dt,
+                self.upper_dt,
+                bounds="()",
+            ),
             dates=DateRange(self.lower_date, self.upper_date),
         )
         data = serializers.serialize("json", [instance])
         dumped = json.loads(data)
-        for field in ("ints", "dates", "timestamps"):
+        for field in ("ints", "dates", "timestamps", "timestamps_closed_bounds"):
             dumped[0]["fields"][field] = json.loads(dumped[0]["fields"][field])
         check = json.loads(self.test_data)
-        for field in ("ints", "dates", "timestamps"):
+        for field in ("ints", "dates", "timestamps", "timestamps_closed_bounds"):
             check[0]["fields"][field] = json.loads(check[0]["fields"][field])
+
         self.assertEqual(dumped, check)
 
     def test_loading(self):
@@ -532,6 +597,10 @@ class TestSerialization(PostgreSQLSimpleTestCase):
         self.assertEqual(
             instance.timestamps, DateTimeTZRange(self.lower_dt, self.upper_dt)
         )
+        self.assertEqual(
+            instance.timestamps_closed_bounds,
+            DateTimeTZRange(self.lower_dt, self.upper_dt, bounds="()"),
+        )
 
     def test_serialize_range_with_null(self):
         instance = RangesModel(ints=NumericRange(None, 10))
@@ -618,17 +687,15 @@ class TestFormField(PostgreSQLSimpleTestCase):
         self.assertHTMLEqual(
             str(form),
             """
-            <tr>
-                <th>
-                <label for="id_field_0">Field:</label>
-                </th>
-                <td>
+            <div>
+                <fieldset>
+                    <legend>Field:</legend>
                     <input id="id_field_0_0" name="field_0_0" type="text">
                     <input id="id_field_0_1" name="field_0_1" type="text">
                     <input id="id_field_1_0" name="field_1_0" type="text">
                     <input id="id_field_1_1" name="field_1_1" type="text">
-                </td>
-            </tr>
+                </fieldset>
+            </div>
         """,
         )
         form = SplitForm(
@@ -658,7 +725,7 @@ class TestFormField(PostgreSQLSimpleTestCase):
             form.as_table(),
             """
             <tr><th>
-            <label for="id_datetime_field_0">Datetime field:</label>
+            <label>Datetime field:</label>
             </th><td>
             <input type="text" name="datetime_field_0" id="id_datetime_field_0">
             <input type="text" name="datetime_field_1" id="id_datetime_field_1">
@@ -679,7 +746,7 @@ class TestFormField(PostgreSQLSimpleTestCase):
             form.as_table(),
             """
             <tr><th>
-            <label for="id_datetime_field_0">Datetime field:</label>
+            <label>Datetime field:</label>
             </th><td>
             <input type="text" name="datetime_field_0"
             value="2010-01-01 11:13:00" id="id_datetime_field_0">
@@ -719,13 +786,13 @@ class TestFormField(PostgreSQLSimpleTestCase):
         self.assertHTMLEqual(
             str(RangeForm()),
             """
-        <tr>
-            <th><label for="id_ints_0">Ints:</label></th>
-            <td>
+        <div>
+            <fieldset>
+                <legend>Ints:</legend>
                 <input id="id_ints_0" name="ints_0" type="number">
                 <input id="id_ints_1" name="ints_1" type="number">
-            </td>
-        </tr>
+            </fieldset>
+        </div>
         """,
         )
 
@@ -922,7 +989,8 @@ class TestFormField(PostgreSQLSimpleTestCase):
         field = pg_forms.DateTimeRangeField()
         value = field.prepare_value(
             DateTimeTZRange(
-                datetime.datetime(2015, 5, 22, 16, 6, 33, tzinfo=timezone.utc), None
+                datetime.datetime(2015, 5, 22, 16, 6, 33, tzinfo=datetime.timezone.utc),
+                None,
             )
         )
         self.assertEqual(value, [datetime.datetime(2015, 5, 22, 18, 6, 33), None])
@@ -947,26 +1015,47 @@ class TestFormField(PostgreSQLSimpleTestCase):
         model_field = pg_fields.IntegerRangeField()
         form_field = model_field.formfield()
         self.assertIsInstance(form_field, pg_forms.IntegerRangeField)
+        self.assertEqual(form_field.range_kwargs, {})
 
     def test_model_field_formfield_biginteger(self):
         model_field = pg_fields.BigIntegerRangeField()
         form_field = model_field.formfield()
         self.assertIsInstance(form_field, pg_forms.IntegerRangeField)
+        self.assertEqual(form_field.range_kwargs, {})
 
     def test_model_field_formfield_float(self):
-        model_field = pg_fields.DecimalRangeField()
+        model_field = pg_fields.DecimalRangeField(default_bounds="()")
         form_field = model_field.formfield()
         self.assertIsInstance(form_field, pg_forms.DecimalRangeField)
+        self.assertEqual(form_field.range_kwargs, {"bounds": "()"})
 
     def test_model_field_formfield_date(self):
         model_field = pg_fields.DateRangeField()
         form_field = model_field.formfield()
         self.assertIsInstance(form_field, pg_forms.DateRangeField)
+        self.assertEqual(form_field.range_kwargs, {})
 
     def test_model_field_formfield_datetime(self):
         model_field = pg_fields.DateTimeRangeField()
         form_field = model_field.formfield()
         self.assertIsInstance(form_field, pg_forms.DateTimeRangeField)
+        self.assertEqual(
+            form_field.range_kwargs,
+            {"bounds": pg_fields.ranges.CANONICAL_RANGE_BOUNDS},
+        )
+
+    def test_model_field_formfield_datetime_default_bounds(self):
+        model_field = pg_fields.DateTimeRangeField(default_bounds="[]")
+        form_field = model_field.formfield()
+        self.assertIsInstance(form_field, pg_forms.DateTimeRangeField)
+        self.assertEqual(form_field.range_kwargs, {"bounds": "[]"})
+
+    def test_model_field_with_default_bounds(self):
+        field = pg_forms.DateTimeRangeField(default_bounds="[]")
+        value = field.clean(["2014-01-01 00:00:00", "2014-02-03 12:13:14"])
+        lower = datetime.datetime(2014, 1, 1, 0, 0, 0)
+        upper = datetime.datetime(2014, 2, 3, 12, 13, 14)
+        self.assertEqual(value, DateTimeTZRange(lower, upper, "[]"))
 
     def test_has_changed(self):
         for field, value in (
@@ -1003,3 +1092,15 @@ class TestWidget(PostgreSQLSimpleTestCase):
             '<input type="text" name="datetimerange_0" value="2006-01-10 07:30:00">'
             '<input type="text" name="datetimerange_1" value="2006-02-12 09:50:00">',
         )
+
+    def test_range_widget_render_tuple_value(self):
+        field = pg_forms.ranges.DateTimeRangeField()
+        dt_range_tuple = (
+            datetime.datetime(2022, 4, 22, 10, 24),
+            datetime.datetime(2022, 5, 12, 9, 25),
+        )
+        self.assertHTMLEqual(
+            field.widget.render("datetimerange", dt_range_tuple),
+            '<input type="text" name="datetimerange_0" value="2022-04-22 10:24:00">'
+            '<input type="text" name="datetimerange_1" value="2022-05-12 09:25:00">',
+        )
