@@ -7,7 +7,8 @@ from django.db.models import Prefetch, QuerySet, prefetch_related_objects
 from django.db.models.query import get_prefetcher
 from django.db.models.sql import Query
 from django.test import TestCase, override_settings
-from django.test.utils import CaptureQueriesContext
+from django.test.utils import CaptureQueriesContext, ignore_warnings
+from django.utils.deprecation import RemovedInDjango50Warning
 
 from .models import (
     Article,
@@ -268,7 +269,7 @@ class PrefetchRelatedTests(TestDataMixin, TestCase):
             list(qs.all())
 
     def test_attribute_error(self):
-        qs = Reader.objects.all().prefetch_related("books_read__xyz")
+        qs = Reader.objects.prefetch_related("books_read__xyz")
         msg = (
             "Cannot find 'xyz' on Book object, 'books_read__xyz' "
             "is an invalid parameter to prefetch_related()"
@@ -368,6 +369,38 @@ class PrefetchRelatedTests(TestDataMixin, TestCase):
             ["Anne", "Charlotte", "Emily", "Jane"],
         )
 
+    def test_m2m_prefetching_iterator_with_chunks(self):
+        with self.assertNumQueries(3):
+            authors = [
+                b.authors.first()
+                for b in Book.objects.prefetch_related("authors").iterator(chunk_size=2)
+            ]
+        self.assertEqual(
+            authors,
+            [self.author1, self.author1, self.author3, self.author4],
+        )
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_m2m_prefetching_iterator_without_chunks(self):
+        # prefetch_related() is ignored.
+        with self.assertNumQueries(5):
+            authors = [
+                b.authors.first()
+                for b in Book.objects.prefetch_related("authors").iterator()
+            ]
+        self.assertEqual(
+            authors,
+            [self.author1, self.author1, self.author3, self.author4],
+        )
+
+    def test_m2m_prefetching_iterator_without_chunks_warning(self):
+        msg = (
+            "Using QuerySet.iterator() after prefetch_related() without "
+            "specifying chunk_size is deprecated."
+        )
+        with self.assertWarnsMessage(RemovedInDjango50Warning, msg):
+            Book.objects.prefetch_related("authors").iterator()
+
 
 class RawQuerySetTests(TestDataMixin, TestCase):
     def test_basic(self):
@@ -839,43 +872,33 @@ class CustomPrefetchTests(TestCase):
         # Test ForwardManyToOneDescriptor.
         houses = House.objects.select_related("owner")
         with self.assertNumQueries(6):
-            rooms = Room.objects.all().prefetch_related("house")
+            rooms = Room.objects.prefetch_related("house")
             lst1 = self.traverse_qs(rooms, [["house", "owner"]])
         with self.assertNumQueries(2):
-            rooms = Room.objects.all().prefetch_related(
-                Prefetch("house", queryset=houses.all())
-            )
+            rooms = Room.objects.prefetch_related(Prefetch("house", queryset=houses))
             lst2 = self.traverse_qs(rooms, [["house", "owner"]])
         self.assertEqual(lst1, lst2)
         with self.assertNumQueries(2):
             houses = House.objects.select_related("owner")
-            rooms = Room.objects.all().prefetch_related(
-                Prefetch("house", queryset=houses.all(), to_attr="house_attr")
+            rooms = Room.objects.prefetch_related(
+                Prefetch("house", queryset=houses, to_attr="house_attr")
             )
             lst2 = self.traverse_qs(rooms, [["house_attr", "owner"]])
         self.assertEqual(lst1, lst2)
-        room = (
-            Room.objects.all()
-            .prefetch_related(
-                Prefetch("house", queryset=houses.filter(address="DoesNotExist"))
-            )
-            .first()
-        )
+        room = Room.objects.prefetch_related(
+            Prefetch("house", queryset=houses.filter(address="DoesNotExist"))
+        ).first()
         with self.assertRaises(ObjectDoesNotExist):
             getattr(room, "house")
-        room = (
-            Room.objects.all()
-            .prefetch_related(
-                Prefetch(
-                    "house",
-                    queryset=houses.filter(address="DoesNotExist"),
-                    to_attr="house_attr",
-                )
+        room = Room.objects.prefetch_related(
+            Prefetch(
+                "house",
+                queryset=houses.filter(address="DoesNotExist"),
+                to_attr="house_attr",
             )
-            .first()
-        )
+        ).first()
         self.assertIsNone(room.house_attr)
-        rooms = Room.objects.all().prefetch_related(
+        rooms = Room.objects.prefetch_related(
             Prefetch("house", queryset=House.objects.only("name"))
         )
         with self.assertNumQueries(2):
@@ -886,20 +909,20 @@ class CustomPrefetchTests(TestCase):
         # Test ReverseOneToOneDescriptor.
         houses = House.objects.select_related("owner")
         with self.assertNumQueries(6):
-            rooms = Room.objects.all().prefetch_related("main_room_of")
+            rooms = Room.objects.prefetch_related("main_room_of")
             lst1 = self.traverse_qs(rooms, [["main_room_of", "owner"]])
         with self.assertNumQueries(2):
-            rooms = Room.objects.all().prefetch_related(
-                Prefetch("main_room_of", queryset=houses.all())
+            rooms = Room.objects.prefetch_related(
+                Prefetch("main_room_of", queryset=houses)
             )
             lst2 = self.traverse_qs(rooms, [["main_room_of", "owner"]])
         self.assertEqual(lst1, lst2)
         with self.assertNumQueries(2):
             rooms = list(
-                Room.objects.all().prefetch_related(
+                Room.objects.prefetch_related(
                     Prefetch(
                         "main_room_of",
-                        queryset=houses.all(),
+                        queryset=houses,
                         to_attr="main_room_of_attr",
                     )
                 )
@@ -1249,7 +1272,7 @@ class MultiTableInheritanceTest(TestCase):
         with self.assertNumQueries(2):
             qs = BookReview.objects.prefetch_related("book")
             titles = [obj.book.title for obj in qs]
-        self.assertEqual(titles, ["Poems", "More poems"])
+        self.assertCountEqual(titles, ["Poems", "More poems"])
 
     def test_m2m_to_inheriting_model(self):
         qs = AuthorWithAge.objects.prefetch_related("books_with_year")
@@ -1313,7 +1336,7 @@ class ForeignKeyToFieldTest(TestCase):
 
     def test_m2m(self):
         with self.assertNumQueries(3):
-            qs = Author.objects.all().prefetch_related("favorite_authors", "favors_me")
+            qs = Author.objects.prefetch_related("favorite_authors", "favors_me")
             favorites = [
                 (
                     [str(i_like) for i_like in author.favorite_authors.all()],
@@ -1589,10 +1612,7 @@ class MultiDbTests(TestCase):
                 % (b.title, ", ".join(a.name for a in b.first_time_authors.all()))
                 for b in B.prefetch_related(prefetch)
             )
-        self.assertEqual(
-            books,
-            "Poems ()\nSense and Sensibility ()\n",
-        )
+        self.assertEqual(books, "Poems ()\n" "Sense and Sensibility ()\n")
 
 
 class Ticket19607Tests(TestCase):
@@ -1851,3 +1871,32 @@ class ReadPrefetchedObjectsCacheTests(TestCase):
         with self.assertNumQueries(4):
             # AuthorWithAge -> Author -> FavoriteAuthors, Book
             self.assertSequenceEqual(authors, [self.author1, self.author2])
+
+
+class NestedPrefetchTests(TestCase):
+    @classmethod
+    def setUpTestData(cls):
+        house = House.objects.create(name="Big house", address="123 Main St")
+        cls.room = Room.objects.create(name="Kitchen", house=house)
+
+    def test_nested_prefetch_is_not_overwritten_by_related_object(self):
+        """
+        The prefetched relationship is used rather than populating the reverse
+        relationship from the parent, when prefetching a set of child objects
+        related to a set of parent objects and the child queryset itself
+        specifies a prefetch back to the parent.
+        """
+        queryset = House.objects.only("name").prefetch_related(
+            Prefetch(
+                "rooms",
+                queryset=Room.objects.prefetch_related(
+                    Prefetch("house", queryset=House.objects.only("address")),
+                ),
+            ),
+        )
+        with self.assertNumQueries(3):
+            house = queryset.first()
+
+        self.assertIs(Room.house.is_cached(self.room), True)
+        with self.assertNumQueries(0):
+            house.rooms.first().house.address
