@@ -1,11 +1,11 @@
 import operator
 
 from django.db import DatabaseError, NotSupportedError, connection
-from django.db.models import Exists, F, IntegerField, OuterRef, Value
+from django.db.models import Exists, F, IntegerField, OuterRef, Subquery, Value
 from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature
 from django.test.utils import CaptureQueriesContext
 
-from .models import Celebrity, Number, ReservedName
+from .models import Author, Celebrity, ExtraInfo, Number, ReservedName
 
 
 @skipUnlessDBFeature("supports_select_union")
@@ -110,7 +110,7 @@ class QuerySetSetOperationTests(TestCase):
         self.assertEqual(len(qs3.union(qs3)), 0)
 
     def test_empty_qs_union_with_ordered_qs(self):
-        qs1 = Number.objects.all().order_by("num")
+        qs1 = Number.objects.order_by("num")
         qs2 = Number.objects.none().union(qs1).order_by("num")
         self.assertEqual(list(qs1), list(qs2))
 
@@ -278,6 +278,71 @@ class QuerySetSetOperationTests(TestCase):
             [reserved_name.pk],
         )
 
+    def test_union_in_subquery(self):
+        ReservedName.objects.bulk_create(
+            [
+                ReservedName(name="rn1", order=8),
+                ReservedName(name="rn2", order=1),
+                ReservedName(name="rn3", order=5),
+            ]
+        )
+        qs1 = Number.objects.filter(num__gt=7, num=OuterRef("order"))
+        qs2 = Number.objects.filter(num__lt=2, num=OuterRef("order"))
+        self.assertCountEqual(
+            ReservedName.objects.annotate(
+                number=Subquery(qs1.union(qs2).values("num")),
+            )
+            .filter(number__isnull=False)
+            .values_list("order", flat=True),
+            [8, 1],
+        )
+
+    def test_union_in_subquery_related_outerref(self):
+        e1 = ExtraInfo.objects.create(value=7, info="e3")
+        e2 = ExtraInfo.objects.create(value=5, info="e2")
+        e3 = ExtraInfo.objects.create(value=1, info="e1")
+        Author.objects.bulk_create(
+            [
+                Author(name="a1", num=1, extra=e1),
+                Author(name="a2", num=3, extra=e2),
+                Author(name="a3", num=2, extra=e3),
+            ]
+        )
+        qs1 = ExtraInfo.objects.order_by().filter(value=OuterRef("num"))
+        qs2 = ExtraInfo.objects.order_by().filter(value__lt=OuterRef("extra__value"))
+        qs = (
+            Author.objects.annotate(
+                info=Subquery(qs1.union(qs2).values("info")[:1]),
+            )
+            .filter(info__isnull=False)
+            .values_list("name", flat=True)
+        )
+        self.assertCountEqual(qs, ["a1", "a2"])
+        # Combined queries don't mutate.
+        self.assertCountEqual(qs, ["a1", "a2"])
+
+    @skipUnlessDBFeature("supports_slicing_ordering_in_compound")
+    def test_union_in_with_ordering(self):
+        qs1 = Number.objects.filter(num__gt=7).order_by("num")
+        qs2 = Number.objects.filter(num__lt=2).order_by("num")
+        self.assertNumbersEqual(
+            Number.objects.exclude(id__in=qs1.union(qs2).values("id")),
+            [2, 3, 4, 5, 6, 7],
+            ordered=False,
+        )
+
+    @skipUnlessDBFeature(
+        "supports_slicing_ordering_in_compound", "allow_sliced_subqueries_with_in"
+    )
+    def test_union_in_with_ordering_and_slice(self):
+        qs1 = Number.objects.filter(num__gt=7).order_by("num")[:1]
+        qs2 = Number.objects.filter(num__lt=2).order_by("-num")[:1]
+        self.assertNumbersEqual(
+            Number.objects.exclude(id__in=qs1.union(qs2).values("id")),
+            [0, 2, 3, 4, 5, 6, 7, 9],
+            ordered=False,
+        )
+
     def test_count_union(self):
         qs1 = Number.objects.filter(num__lte=1).values("num")
         qs2 = Number.objects.filter(num__gte=2, num__lte=3).values("num")
@@ -471,3 +536,25 @@ class QuerySetSetOperationTests(TestCase):
             with self.subTest(combinator=combinator):
                 with self.assertRaisesMessage(NotSupportedError, msg % combinator):
                     getattr(qs, combinator)(qs).get(num=2)
+
+    def test_operator_on_combined_qs_error(self):
+        qs = Number.objects.all()
+        msg = "Cannot use %s operator with combined queryset."
+        combinators = ["union"]
+        if connection.features.supports_select_difference:
+            combinators.append("difference")
+        if connection.features.supports_select_intersection:
+            combinators.append("intersection")
+        operators = [
+            ("|", operator.or_),
+            ("&", operator.and_),
+            ("^", operator.xor),
+        ]
+        for combinator in combinators:
+            combined_qs = getattr(qs, combinator)(qs)
+            for operator_, operator_func in operators:
+                with self.subTest(combinator=combinator):
+                    with self.assertRaisesMessage(TypeError, msg % operator_):
+                        operator_func(qs, combined_qs)
+                    with self.assertRaisesMessage(TypeError, msg % operator_):
+                        operator_func(combined_qs, qs)
