@@ -32,21 +32,23 @@ from django.test import (
     SimpleTestCase,
     TestCase,
     TransactionTestCase,
-    ignore_warnings,
     skipIfDBFeature,
     skipUnlessDBFeature,
 )
 from django.test.html import HTMLParseError, parse_html
+from django.test.testcases import DatabaseOperationForbidden
 from django.test.utils import (
     CaptureQueriesContext,
     TestContextDecorator,
+    ignore_warnings,
     isolate_apps,
     override_settings,
     setup_test_environment,
 )
 from django.urls import NoReverseMatch, path, reverse, reverse_lazy
-from django.utils.deprecation import RemovedInDjango41Warning
+from django.utils.deprecation import RemovedInDjango50Warning
 from django.utils.log import DEFAULT_LOGGING
+from django.utils.version import PY311
 
 from .models import Car, Person, PossessedCar
 from .views import empty_response
@@ -99,9 +101,11 @@ class SkippingTestCase(SimpleTestCase):
             SkipTestCase("test_foo").test_foo,
             ValueError,
             "skipUnlessDBFeature cannot be used on test_foo (test_utils.tests."
-            "SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase) "
+            "SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase%s) "
             "as SkippingTestCase.test_skip_unless_db_feature.<locals>.SkipTestCase "
-            "doesn't allow queries against the 'default' database.",
+            "doesn't allow queries against the 'default' database."
+            # Python 3.11 uses fully qualified test name in the output.
+            % (".test_foo" if PY311 else ""),
         )
 
     def test_skip_if_db_feature(self):
@@ -144,9 +148,11 @@ class SkippingTestCase(SimpleTestCase):
             SkipTestCase("test_foo").test_foo,
             ValueError,
             "skipIfDBFeature cannot be used on test_foo (test_utils.tests."
-            "SkippingTestCase.test_skip_if_db_feature.<locals>.SkipTestCase) "
+            "SkippingTestCase.test_skip_if_db_feature.<locals>.SkipTestCase%s) "
             "as SkippingTestCase.test_skip_if_db_feature.<locals>.SkipTestCase "
-            "doesn't allow queries against the 'default' database.",
+            "doesn't allow queries against the 'default' database."
+            # Python 3.11 uses fully qualified test name in the output.
+            % (".test_foo" if PY311 else ""),
         )
 
 
@@ -269,37 +275,37 @@ class AssertQuerysetEqualTests(TestCase):
 
     def test_ordered(self):
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("name"),
+            Person.objects.order_by("name"),
             [self.p1, self.p2],
         )
 
     def test_unordered(self):
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("name"), [self.p2, self.p1], ordered=False
+            Person.objects.order_by("name"), [self.p2, self.p1], ordered=False
         )
 
     def test_queryset(self):
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("name"),
-            Person.objects.all().order_by("name"),
+            Person.objects.order_by("name"),
+            Person.objects.order_by("name"),
         )
 
     def test_flat_values_list(self):
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("name").values_list("name", flat=True),
+            Person.objects.order_by("name").values_list("name", flat=True),
             ["p1", "p2"],
         )
 
     def test_transform(self):
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("name"),
+            Person.objects.order_by("name"),
             [self.p1.pk, self.p2.pk],
             transform=lambda x: x.pk,
         )
 
     def test_repr_transform(self):
         self.assertQuerysetEqual(
-            Person.objects.all().order_by("name"),
+            Person.objects.order_by("name"),
             [repr(self.p1), repr(self.p2)],
             transform=repr,
         )
@@ -376,32 +382,6 @@ class AssertQuerysetEqualTests(TestCase):
             self.assertIn(name, exception_msg)
 
 
-class AssertQuerysetEqualDeprecationTests(TestCase):
-    @classmethod
-    def setUpTestData(cls):
-        cls.p1 = Person.objects.create(name="p1")
-        cls.p2 = Person.objects.create(name="p2")
-
-    @ignore_warnings(category=RemovedInDjango41Warning)
-    def test_str_values(self):
-        self.assertQuerysetEqual(
-            Person.objects.all().order_by("name"),
-            [repr(self.p1), repr(self.p2)],
-        )
-
-    def test_str_values_warning(self):
-        msg = (
-            "In Django 4.1, repr() will not be called automatically on a "
-            "queryset when compared to string values. Set an explicit "
-            "'transform' to silence this warning."
-        )
-        with self.assertRaisesMessage(RemovedInDjango41Warning, msg):
-            self.assertQuerysetEqual(
-                Person.objects.all().order_by("name"),
-                [repr(self.p1), repr(self.p2)],
-            )
-
-
 @override_settings(ROOT_URLCONF="test_utils.urls")
 class CaptureQueriesContextManagerTests(TestCase):
     @classmethod
@@ -539,7 +519,7 @@ class AssertTemplateUsedContextManagerTests(SimpleTestCase):
             pass
 
     def test_error_message(self):
-        msg = "template_used/base.html was not rendered. No template was rendered."
+        msg = "No templates used to render the response"
         with self.assertRaisesMessage(AssertionError, msg):
             with self.assertTemplateUsed("template_used/base.html"):
                 pass
@@ -549,8 +529,8 @@ class AssertTemplateUsedContextManagerTests(SimpleTestCase):
                 pass
 
         msg2 = (
-            "template_used/base.html was not rendered. Following templates "
-            "were rendered: template_used/alternative.html"
+            "Template 'template_used/base.html' was not a template used to render "
+            "the response. Actual template(s) used: template_used/alternative.html"
         )
         with self.assertRaisesMessage(AssertionError, msg2):
             with self.assertTemplateUsed("template_used/base.html"):
@@ -562,6 +542,47 @@ class AssertTemplateUsedContextManagerTests(SimpleTestCase):
             response = self.client.get("/test_utils/no_template_used/")
             self.assertTemplateUsed(response, "template_used/base.html")
 
+    def test_msg_prefix(self):
+        msg_prefix = "Prefix"
+        msg = f"{msg_prefix}: No templates used to render the response"
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertTemplateUsed(
+                "template_used/base.html", msg_prefix=msg_prefix
+            ):
+                pass
+
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertTemplateUsed(
+                template_name="template_used/base.html",
+                msg_prefix=msg_prefix,
+            ):
+                pass
+
+        msg = (
+            f"{msg_prefix}: Template 'template_used/base.html' was not a "
+            f"template used to render the response. Actual template(s) used: "
+            f"template_used/alternative.html"
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertTemplateUsed(
+                "template_used/base.html", msg_prefix=msg_prefix
+            ):
+                render_to_string("template_used/alternative.html")
+
+    def test_count(self):
+        with self.assertTemplateUsed("template_used/base.html", count=2):
+            render_to_string("template_used/base.html")
+            render_to_string("template_used/base.html")
+
+        msg = (
+            "Template 'template_used/base.html' was expected to be rendered "
+            "3 time(s) but was actually rendered 2 time(s)."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            with self.assertTemplateUsed("template_used/base.html", count=3):
+                render_to_string("template_used/base.html")
+                render_to_string("template_used/base.html")
+
     def test_failure(self):
         msg = "response and/or template_name argument must be provided"
         with self.assertRaisesMessage(TypeError, msg):
@@ -582,8 +603,9 @@ class AssertTemplateUsedContextManagerTests(SimpleTestCase):
                 pass
 
         msg = (
-            "template_used/base.html was not rendered. Following "
-            "templates were rendered: template_used/alternative.html"
+            "Template 'template_used/base.html' was not a template used to "
+            "render the response. Actual template(s) used: "
+            "template_used/alternative.html"
         )
         with self.assertRaisesMessage(AssertionError, msg):
             with self.assertTemplateUsed("template_used/base.html"):
@@ -591,14 +613,10 @@ class AssertTemplateUsedContextManagerTests(SimpleTestCase):
 
     def test_assert_used_on_http_response(self):
         response = HttpResponse()
-        error_msg = (
-            "assertTemplateUsed() and assertTemplateNotUsed() are only "
-            "usable on responses fetched using the Django test Client."
-        )
-        with self.assertRaisesMessage(ValueError, error_msg):
+        msg = "%s() is only usable on responses fetched using the Django test Client."
+        with self.assertRaisesMessage(ValueError, msg % "assertTemplateUsed"):
             self.assertTemplateUsed(response, "template.html")
-
-        with self.assertRaisesMessage(ValueError, error_msg):
+        with self.assertRaisesMessage(ValueError, msg % "assertTemplateNotUsed"):
             self.assertTemplateNotUsed(response, "template.html")
 
 
@@ -922,6 +940,11 @@ class HTMLEqualTests(SimpleTestCase):
         dom2 = parse_html("<a><b/><b/></a><b/><b/>")
         self.assertEqual(dom2.count(dom1), 2)
 
+    def test_root_element_escaped_html(self):
+        html = "&lt;br&gt;"
+        parsed = parse_html(html)
+        self.assertEqual(str(parsed), html)
+
     def test_parsing_errors(self):
         with self.assertRaises(AssertionError):
             self.assertHTMLEqual("<p>", "")
@@ -936,6 +959,13 @@ class HTMLEqualTests(SimpleTestCase):
         with self.assertRaises(HTMLParseError):
             parse_html("</p>")
 
+    def test_escaped_html_errors(self):
+        msg = "<p>\n<foo>\n</p> != <p>\n&lt;foo&gt;\n</p>\n"
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertHTMLEqual("<p><foo></p>", "<p>&lt;foo&gt;</p>")
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertHTMLEqual("<p><foo></p>", "<p>&#60;foo&#62;</p>")
+
     def test_contains_html(self):
         response = HttpResponse(
             """<body>
@@ -1301,6 +1331,25 @@ class TestForm(Form):
             raise ValidationError("invalid value")
         return value
 
+    def clean(self):
+        if self.cleaned_data.get("field") == "invalid_non_field":
+            raise ValidationError("non-field error")
+        return self.cleaned_data
+
+    @classmethod
+    def _get_cleaned_form(cls, field_value):
+        form = cls({"field": field_value})
+        form.full_clean()
+        return form
+
+    @classmethod
+    def valid(cls):
+        return cls._get_cleaned_form("valid")
+
+    @classmethod
+    def invalid(cls, nonfield=False):
+        return cls._get_cleaned_form("invalid_non_field" if nonfield else "invalid")
+
 
 class TestFormset(formset_factory(TestForm)):
     @classmethod
@@ -1316,22 +1365,600 @@ class TestFormset(formset_factory(TestForm)):
         return formset
 
     @classmethod
-    def invalid(cls):
-        return cls._get_cleaned_formset("invalid")
+    def valid(cls):
+        return cls._get_cleaned_formset("valid")
+
+    @classmethod
+    def invalid(cls, nonfield=False, nonform=False):
+        if nonform:
+            formset = cls({}, error_messages={"missing_management_form": "error"})
+            formset.full_clean()
+            return formset
+        return cls._get_cleaned_formset("invalid_non_field" if nonfield else "invalid")
+
+
+class AssertFormErrorTests(SimpleTestCase):
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_non_client_response(self):
+        msg = (
+            "assertFormError() is only usable on responses fetched using the "
+            "Django test Client."
+        )
+        response = HttpResponse()
+        with self.assertRaisesMessage(ValueError, msg):
+            self.assertFormError(response, "form", "field", "invalid value")
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_response_with_no_context(self):
+        msg = "Response did not use any contexts to render the response"
+        response = mock.Mock(context=[])
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormError(response, "form", "field", "invalid value")
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(
+                response,
+                "form",
+                "field",
+                "invalid value",
+                msg_prefix=msg_prefix,
+            )
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_form_not_in_context(self):
+        msg = "The form 'form' was not used to render the response"
+        response = mock.Mock(context=[{}])
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormError(response, "form", "field", "invalid value")
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(
+                response, "form", "field", "invalid value", msg_prefix=msg_prefix
+            )
+
+    def test_single_error(self):
+        self.assertFormError(TestForm.invalid(), "field", "invalid value")
+
+    def test_error_list(self):
+        self.assertFormError(TestForm.invalid(), "field", ["invalid value"])
+
+    def test_empty_errors_valid_form(self):
+        self.assertFormError(TestForm.valid(), "field", [])
+
+    def test_empty_errors_valid_form_non_field_errors(self):
+        self.assertFormError(TestForm.valid(), None, [])
+
+    def test_field_not_in_form(self):
+        msg = (
+            "The form <TestForm bound=True, valid=False, fields=(field)> does not "
+            "contain the field 'other_field'."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormError(TestForm.invalid(), "other_field", "invalid value")
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(
+                TestForm.invalid(),
+                "other_field",
+                "invalid value",
+                msg_prefix=msg_prefix,
+            )
+
+    def test_field_with_no_errors(self):
+        msg = (
+            "The errors of field 'field' on form <TestForm bound=True, valid=True, "
+            "fields=(field)> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormError(TestForm.valid(), "field", "invalid value")
+        self.assertIn("[] != ['invalid value']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(
+                TestForm.valid(), "field", "invalid value", msg_prefix=msg_prefix
+            )
+
+    def test_field_with_different_error(self):
+        msg = (
+            "The errors of field 'field' on form <TestForm bound=True, valid=False, "
+            "fields=(field)> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormError(TestForm.invalid(), "field", "other error")
+        self.assertIn("['invalid value'] != ['other error']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(
+                TestForm.invalid(), "field", "other error", msg_prefix=msg_prefix
+            )
+
+    def test_unbound_form(self):
+        msg = (
+            "The form <TestForm bound=False, valid=Unknown, fields=(field)> is not "
+            "bound, it will never have any errors."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormError(TestForm(), "field", [])
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(TestForm(), "field", [], msg_prefix=msg_prefix)
+
+    def test_empty_errors_invalid_form(self):
+        msg = (
+            "The errors of field 'field' on form <TestForm bound=True, valid=False, "
+            "fields=(field)> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormError(TestForm.invalid(), "field", [])
+        self.assertIn("['invalid value'] != []", str(ctx.exception))
+
+    def test_non_field_errors(self):
+        self.assertFormError(TestForm.invalid(nonfield=True), None, "non-field error")
+
+    def test_different_non_field_errors(self):
+        msg = (
+            "The non-field errors of form <TestForm bound=True, valid=False, "
+            "fields=(field)> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormError(
+                TestForm.invalid(nonfield=True), None, "other non-field error"
+            )
+        self.assertIn(
+            "['non-field error'] != ['other non-field error']", str(ctx.exception)
+        )
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormError(
+                TestForm.invalid(nonfield=True),
+                None,
+                "other non-field error",
+                msg_prefix=msg_prefix,
+            )
 
 
 class AssertFormsetErrorTests(SimpleTestCase):
-    def test_formset_named_form(self):
-        formset = TestFormset.invalid()
-        # The mocked context emulates the template-based rendering of the
-        # formset.
-        response = mock.Mock(
-            context=[
-                {"form": formset},
-                {"form": formset.management_form},
-            ]
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_non_client_response(self):
+        msg = (
+            "assertFormsetError() is only usable on responses fetched using "
+            "the Django test Client."
         )
-        self.assertFormsetError(response, "form", 0, "field", "invalid value")
+        response = HttpResponse()
+        with self.assertRaisesMessage(ValueError, msg):
+            self.assertFormsetError(response, "formset", 0, "field", "invalid value")
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_response_with_no_context(self):
+        msg = "Response did not use any contexts to render the response"
+        response = mock.Mock(context=[])
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(response, "formset", 0, "field", "invalid value")
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_formset_not_in_context(self):
+        msg = "The formset 'formset' was not used to render the response"
+        response = mock.Mock(context=[{}])
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(response, "formset", 0, "field", "invalid value")
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                response, "formset", 0, "field", "invalid value", msg_prefix=msg_prefix
+            )
+
+    def test_single_error(self):
+        self.assertFormsetError(TestFormset.invalid(), 0, "field", "invalid value")
+
+    def test_error_list(self):
+        self.assertFormsetError(TestFormset.invalid(), 0, "field", ["invalid value"])
+
+    def test_empty_errors_valid_formset(self):
+        self.assertFormsetError(TestFormset.valid(), 0, "field", [])
+
+    def test_multiple_forms(self):
+        formset = TestFormset(
+            {
+                "form-TOTAL_FORMS": "2",
+                "form-INITIAL_FORMS": "0",
+                "form-0-field": "valid",
+                "form-1-field": "invalid",
+            }
+        )
+        formset.full_clean()
+        self.assertFormsetError(formset, 0, "field", [])
+        self.assertFormsetError(formset, 1, "field", ["invalid value"])
+
+    def test_field_not_in_form(self):
+        msg = (
+            "The form 0 of formset <TestFormset: bound=True valid=False total_forms=1> "
+            "does not contain the field 'other_field'."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(
+                TestFormset.invalid(), 0, "other_field", "invalid value"
+            )
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.invalid(),
+                0,
+                "other_field",
+                "invalid value",
+                msg_prefix=msg_prefix,
+            )
+
+    def test_field_with_no_errors(self):
+        msg = (
+            "The errors of field 'field' on form 0 of formset <TestFormset: bound=True "
+            "valid=True total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(TestFormset.valid(), 0, "field", "invalid value")
+        self.assertIn("[] != ['invalid value']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.valid(), 0, "field", "invalid value", msg_prefix=msg_prefix
+            )
+
+    def test_field_with_different_error(self):
+        msg = (
+            "The errors of field 'field' on form 0 of formset <TestFormset: bound=True "
+            "valid=False total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(TestFormset.invalid(), 0, "field", "other error")
+        self.assertIn("['invalid value'] != ['other error']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.invalid(), 0, "field", "other error", msg_prefix=msg_prefix
+            )
+
+    def test_unbound_formset(self):
+        msg = (
+            "The formset <TestFormset: bound=False valid=Unknown total_forms=1> is not "
+            "bound, it will never have any errors."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(TestFormset(), 0, "field", [])
+
+    def test_empty_errors_invalid_formset(self):
+        msg = (
+            "The errors of field 'field' on form 0 of formset <TestFormset: bound=True "
+            "valid=False total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(TestFormset.invalid(), 0, "field", [])
+        self.assertIn("['invalid value'] != []", str(ctx.exception))
+
+    def test_non_field_errors(self):
+        self.assertFormsetError(
+            TestFormset.invalid(nonfield=True), 0, None, "non-field error"
+        )
+
+    def test_different_non_field_errors(self):
+        msg = (
+            "The non-field errors of form 0 of formset <TestFormset: bound=True "
+            "valid=False total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(
+                TestFormset.invalid(nonfield=True), 0, None, "other non-field error"
+            )
+        self.assertIn(
+            "['non-field error'] != ['other non-field error']", str(ctx.exception)
+        )
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.invalid(nonfield=True),
+                0,
+                None,
+                "other non-field error",
+                msg_prefix=msg_prefix,
+            )
+
+    def test_no_non_field_errors(self):
+        msg = (
+            "The non-field errors of form 0 of formset <TestFormset: bound=True "
+            "valid=False total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(TestFormset.invalid(), 0, None, "non-field error")
+        self.assertIn("[] != ['non-field error']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.invalid(), 0, None, "non-field error", msg_prefix=msg_prefix
+            )
+
+    def test_non_form_errors(self):
+        self.assertFormsetError(TestFormset.invalid(nonform=True), None, None, "error")
+
+    def test_different_non_form_errors(self):
+        msg = (
+            "The non-form errors of formset <TestFormset: bound=True valid=False "
+            "total_forms=0> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(
+                TestFormset.invalid(nonform=True), None, None, "other error"
+            )
+        self.assertIn("['error'] != ['other error']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.invalid(nonform=True),
+                None,
+                None,
+                "other error",
+                msg_prefix=msg_prefix,
+            )
+
+    def test_no_non_form_errors(self):
+        msg = (
+            "The non-form errors of formset <TestFormset: bound=True valid=False "
+            "total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg) as ctx:
+            self.assertFormsetError(TestFormset.invalid(), None, None, "error")
+        self.assertIn("[] != ['error']", str(ctx.exception))
+        msg_prefix = "Custom prefix"
+        with self.assertRaisesMessage(AssertionError, f"{msg_prefix}: {msg}"):
+            self.assertFormsetError(
+                TestFormset.invalid(),
+                None,
+                None,
+                "error",
+                msg_prefix=msg_prefix,
+            )
+
+    def test_non_form_errors_with_field(self):
+        msg = "You must use field=None with form_index=None."
+        with self.assertRaisesMessage(ValueError, msg):
+            self.assertFormsetError(
+                TestFormset.invalid(nonform=True), None, "field", "error"
+            )
+
+    def test_form_index_too_big(self):
+        msg = (
+            "The formset <TestFormset: bound=True valid=False total_forms=1> only has "
+            "1 form."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(TestFormset.invalid(), 2, "field", "error")
+
+    def test_form_index_too_big_plural(self):
+        formset = TestFormset(
+            {
+                "form-TOTAL_FORMS": "2",
+                "form-INITIAL_FORMS": "0",
+                "form-0-field": "valid",
+                "form-1-field": "valid",
+            }
+        )
+        formset.full_clean()
+        msg = (
+            "The formset <TestFormset: bound=True valid=True total_forms=2> only has 2 "
+            "forms."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(formset, 2, "field", "error")
+
+
+# RemovedInDjango50Warning
+class AssertFormErrorDeprecationTests(SimpleTestCase):
+    """
+    Exhaustively test all possible combinations of args/kwargs for the old
+    signature.
+    """
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_assert_form_error_errors_none(self):
+        msg = (
+            "The errors of field 'field' on form <TestForm bound=True, valid=False, "
+            "fields=(field)> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormError(TestForm.invalid(), "field", None)
+
+    def test_assert_form_error_errors_none_warning(self):
+        msg = (
+            "Passing errors=None to assertFormError() is deprecated, use "
+            "errors=[] instead."
+        )
+        with self.assertWarnsMessage(RemovedInDjango50Warning, msg):
+            self.assertFormError(TestForm.valid(), "field", None)
+
+    def _assert_form_error_old_api_cases(self, form, field, errors, msg_prefix):
+        response = mock.Mock(context=[{"form": TestForm.invalid()}])
+        return (
+            ((response, form, field, errors), {}),
+            ((response, form, field, errors, msg_prefix), {}),
+            ((response, form, field, errors), {"msg_prefix": msg_prefix}),
+            ((response, form, field), {"errors": errors}),
+            ((response, form, field), {"errors": errors, "msg_prefix": msg_prefix}),
+            ((response, form), {"field": field, "errors": errors}),
+            (
+                (response, form),
+                {"field": field, "errors": errors, "msg_prefix": msg_prefix},
+            ),
+            ((response,), {"form": form, "field": field, "errors": errors}),
+            (
+                (response,),
+                {
+                    "form": form,
+                    "field": field,
+                    "errors": errors,
+                    "msg_prefix": msg_prefix,
+                },
+            ),
+            (
+                (),
+                {"response": response, "form": form, "field": field, "errors": errors},
+            ),
+            (
+                (),
+                {
+                    "response": response,
+                    "form": form,
+                    "field": field,
+                    "errors": errors,
+                    "msg_prefix": msg_prefix,
+                },
+            ),
+        )
+
+    def test_assert_form_error_old_api(self):
+        deprecation_msg = (
+            "Passing response to assertFormError() is deprecated. Use the form object "
+            "directly: assertFormError(response.context['form'], 'field', ...)"
+        )
+        for args, kwargs in self._assert_form_error_old_api_cases(
+            form="form",
+            field="field",
+            errors=["invalid value"],
+            msg_prefix="Custom prefix",
+        ):
+            with self.subTest(args=args, kwargs=kwargs):
+                with self.assertWarnsMessage(RemovedInDjango50Warning, deprecation_msg):
+                    self.assertFormError(*args, **kwargs)
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_assert_form_error_old_api_assertion_error(self):
+        for args, kwargs in self._assert_form_error_old_api_cases(
+            form="form",
+            field="field",
+            errors=["other error"],
+            msg_prefix="Custom prefix",
+        ):
+            with self.subTest(args=args, kwargs=kwargs):
+                with self.assertRaises(AssertionError):
+                    self.assertFormError(*args, **kwargs)
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_assert_formset_error_errors_none(self):
+        msg = (
+            "The errors of field 'field' on form 0 of formset <TestFormset: bound=True "
+            "valid=False total_forms=1> don't match."
+        )
+        with self.assertRaisesMessage(AssertionError, msg):
+            self.assertFormsetError(TestFormset.invalid(), 0, "field", None)
+
+    def test_assert_formset_error_errors_none_warning(self):
+        msg = (
+            "Passing errors=None to assertFormsetError() is deprecated, use "
+            "errors=[] instead."
+        )
+        with self.assertWarnsMessage(RemovedInDjango50Warning, msg):
+            self.assertFormsetError(TestFormset.valid(), 0, "field", None)
+
+    def _assert_formset_error_old_api_cases(
+        self, formset, form_index, field, errors, msg_prefix
+    ):
+        response = mock.Mock(context=[{"formset": TestFormset.invalid()}])
+        return (
+            ((response, formset, form_index, field, errors), {}),
+            ((response, formset, form_index, field, errors, msg_prefix), {}),
+            (
+                (response, formset, form_index, field, errors),
+                {"msg_prefix": msg_prefix},
+            ),
+            ((response, formset, form_index, field), {"errors": errors}),
+            (
+                (response, formset, form_index, field),
+                {"errors": errors, "msg_prefix": msg_prefix},
+            ),
+            ((response, formset, form_index), {"field": field, "errors": errors}),
+            (
+                (response, formset, form_index),
+                {"field": field, "errors": errors, "msg_prefix": msg_prefix},
+            ),
+            (
+                (response, formset),
+                {"form_index": form_index, "field": field, "errors": errors},
+            ),
+            (
+                (response, formset),
+                {
+                    "form_index": form_index,
+                    "field": field,
+                    "errors": errors,
+                    "msg_prefix": msg_prefix,
+                },
+            ),
+            (
+                (response,),
+                {
+                    "formset": formset,
+                    "form_index": form_index,
+                    "field": field,
+                    "errors": errors,
+                },
+            ),
+            (
+                (response,),
+                {
+                    "formset": formset,
+                    "form_index": form_index,
+                    "field": field,
+                    "errors": errors,
+                    "msg_prefix": msg_prefix,
+                },
+            ),
+            (
+                (),
+                {
+                    "response": response,
+                    "formset": formset,
+                    "form_index": form_index,
+                    "field": field,
+                    "errors": errors,
+                },
+            ),
+            (
+                (),
+                {
+                    "response": response,
+                    "formset": formset,
+                    "form_index": form_index,
+                    "field": field,
+                    "errors": errors,
+                    "msg_prefix": msg_prefix,
+                },
+            ),
+        )
+
+    def test_assert_formset_error_old_api(self):
+        deprecation_msg = (
+            "Passing response to assertFormsetError() is deprecated. Use the formset "
+            "object directly: assertFormsetError(response.context['formset'], 0, ...)"
+        )
+        for args, kwargs in self._assert_formset_error_old_api_cases(
+            formset="formset",
+            form_index=0,
+            field="field",
+            errors=["invalid value"],
+            msg_prefix="Custom prefix",
+        ):
+            with self.subTest(args=args, kwargs=kwargs):
+                with self.assertWarnsMessage(RemovedInDjango50Warning, deprecation_msg):
+                    self.assertFormsetError(*args, **kwargs)
+
+    @ignore_warnings(category=RemovedInDjango50Warning)
+    def test_assert_formset_error_old_api_assertion_error(self):
+        for args, kwargs in self._assert_formset_error_old_api_cases(
+            formset="formset",
+            form_index=0,
+            field="field",
+            errors=["other error"],
+            msg_prefix="Custom prefix",
+        ):
+            with self.subTest(args=args, kwargs=kwargs):
+                with self.assertRaises(AssertionError):
+                    self.assertFormsetError(*args, **kwargs)
 
 
 class FirstUrls:
@@ -1439,7 +2066,7 @@ class OverrideSettingsTests(SimpleTestCase):
 
     def test_override_database_routers(self):
         """
-        Overriding DATABASE_ROUTERS should update the master router.
+        Overriding DATABASE_ROUTERS should update the base router.
         """
         test_routers = [object()]
         with self.settings(DATABASE_ROUTERS=test_routers):
@@ -1499,6 +2126,7 @@ class OverrideSettingsTests(SimpleTestCase):
             self.assertIn(expected_location, finder.locations)
 
 
+@skipUnlessDBFeature("supports_transactions")
 class TestBadSetUpTestData(TestCase):
     """
     An exception in setUpTestData() shouldn't leak a transaction which would
@@ -1533,6 +2161,7 @@ class TestBadSetUpTestData(TestCase):
         self.assertFalse(self._in_atomic_block)
 
 
+@skipUnlessDBFeature("supports_transactions")
 class CaptureOnCommitCallbacksTests(TestCase):
     databases = {"default", "other"}
     callback_called = False
@@ -1666,9 +2295,9 @@ class DisallowedDatabaseQueriesTests(SimpleTestCase):
             "test_utils.tests.DisallowedDatabaseQueriesTests.databases to "
             "silence this failure."
         )
-        with self.assertRaisesMessage(AssertionError, expected_message):
+        with self.assertRaisesMessage(DatabaseOperationForbidden, expected_message):
             connection.connect()
-        with self.assertRaisesMessage(AssertionError, expected_message):
+        with self.assertRaisesMessage(DatabaseOperationForbidden, expected_message):
             connection.temporary_connection()
 
     def test_disallowed_database_queries(self):
@@ -1679,7 +2308,7 @@ class DisallowedDatabaseQueriesTests(SimpleTestCase):
             "test_utils.tests.DisallowedDatabaseQueriesTests.databases to "
             "silence this failure."
         )
-        with self.assertRaisesMessage(AssertionError, expected_message):
+        with self.assertRaisesMessage(DatabaseOperationForbidden, expected_message):
             Car.objects.first()
 
     def test_disallowed_database_chunked_cursor_queries(self):
@@ -1690,7 +2319,7 @@ class DisallowedDatabaseQueriesTests(SimpleTestCase):
             "test_utils.tests.DisallowedDatabaseQueriesTests.databases to "
             "silence this failure."
         )
-        with self.assertRaisesMessage(AssertionError, expected_message):
+        with self.assertRaisesMessage(DatabaseOperationForbidden, expected_message):
             next(Car.objects.iterator())
 
 
